package jp.agentec.sinaburocast.action.user;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.annotation.NoLogin;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.ServletUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.dto.EnqResultDto;
import jp.agentec.sinaburocast.entity.Enquete;
import jp.agentec.sinaburocast.form.user.EnqResultListForm;
import jp.agentec.sinaburocast.helper.KeyHelper;
import jp.agentec.sinaburocast.service.EnqueteService;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.seasar.framework.util.InputStreamUtil;
import org.seasar.framework.util.OutputStreamUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.RequestUtil;

/**
 * アンケート結果処理
 *
 * @author kim
 *
 */
@NoLogin
public class EnqResultListAction extends AbstractAction {

	private final Logger logger = Logger.getLogger(getClass());

	public EnqueteService enqueteService;

	@ActionForm
	public EnqResultListForm enqResultListForm;

	public KeyHelper keyHelper;

	/**
	 * 初期表示
	 *
	 * @return
	 * @throws ParseException
	 */
	@Execute(validator = false)
	public String index() throws ParseException,IOException {
		session.removeAttribute("enqRsultList");
		session.removeAttribute("enqRsultLastList");

		List<EnqResultDto> enqlist = new ArrayList<EnqResultDto>();
		List<EnqResultDto> enqLastlist = new ArrayList<EnqResultDto>();

		List<Enquete> enqueteInfo = enqueteService.findEnqueteSummaryById();

		if (CollectionUtils.isEmpty(enqueteInfo)) {
			session.setAttribute("enqRsultList", enqlist);
			session.setAttribute("enqRsultLastList", enqLastlist);
			return "/user/enqResultList/enqResultList.html";
		}
		String thisYyyy = enqueteInfo.get(0).startDate.substring(0, 4);

		enqResultListForm.enqYear = thisYyyy;


		ArrayList<String> yearList = new ArrayList<String>(); 
		//リスト作成
		for (int i = 0; i < enqueteInfo.size(); i++) {

			Enquete list = enqueteInfo.get(i);

			if (list.startDate.substring(0, 4).equals(thisYyyy)) {
				EnqResultDto dto = new EnqResultDto();
				dto.enqTitle = list.enqueteName;
				try {
					dto.enqId = URLEncoder.encode(keyHelper.encryptEnqueteId(list.enqueteId), "utf-8");
					dto.startDay = SinaburoUtil.convertStringDateToFormat(list.startDate, "/");
					dto.endDay = SinaburoUtil.convertStringDateToFormat(list.endDate, "/");
					enqlist.add(dto);
				} catch (Exception e) {
					logger.error("Encrypt failed. enqueteId：" + list.enqueteId + e.toString());
				}
			}else {
				EnqResultDto dto1 = new EnqResultDto();
				dto1.enqYear = list.startDate.substring(0, 4);
				try {
					dto1.enqId = URLEncoder.encode(keyHelper.encryptEnqueteId(list.enqueteId), "utf-8");
					if(!yearList.contains(dto1.enqYear)){
						enqLastlist.add(dto1);
						yearList.add(dto1.enqYear);
					}
				} catch (Exception e) {
					logger.error("Encrypt failed. enqueteId：" + list.enqueteId + e.toString());
				}
			}
		}
		session.setAttribute("enqRsultList", enqlist);
		session.setAttribute("enqRsultLastList", enqLastlist);

		return "/user/enqResultList/enqResultList.html";
	}

	/**
	 * 今年のアンケートリンク
	 *
	 * @return
	 */
	@Execute(validator = false)
	public String enqLink() throws Exception{
		String enqId = null;

		try {
			enqId = keyHelper.decryptEnqueteId(enqResultListForm.enqId);
		} catch (Exception e) {
			logger.warn("Decrypt failed. enqueteId：" + enqResultListForm.enqId + e.toString());
			response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
			return null;
		}

		Enquete enquete = enqueteService.findById(Integer.parseInt(enqId));
		if(getAdmin() == null && enquete.enqueteType ==SinaburoConstant.EnqueteType.WEB_ENQUETE && enquete.privateFlg){
			String ipAddress = ServletUtil.getRemoteAddr(RequestUtil.getRequest());
			if(!SinaburoUtil.isAllowedSrc(ipAddress, PropertyUtil.getList("IP_ALLOWED_SRC"))){
				response.sendError(HttpServletResponse.SC_NOT_FOUND);
		    	return null;
			}
		}
		StringBuilder path = new StringBuilder(PropertyUtil.getString("ENQUETE_RESULT"));

		if (StringUtil.equals(enqResultListForm.type, "PC")) {
			path = path.append("/user/enquete" + enqId + ".html");
		} else if (StringUtil.equals(enqResultListForm.type, "FP")) {
			path = path.append("/mb/enquete" + enqId + ".html");
		} else if (StringUtil.equals(enqResultListForm.type, "SP")) {
			path = path.append("/sp/enquete" + enqId + ".html");
		} else {
			path = path.append("/user/enquete" + enqId + ".html");
		}

		InputStream is = new FileInputStream(path.toString());
		OutputStream os = response.getOutputStream();
		response.setContentType("text/html;charset=Shift_JIS");
		try {
			InputStreamUtil.copy(is, os);
			OutputStreamUtil.flush(os);
		} finally {
			InputStreamUtil.close(is);
			OutputStreamUtil.close(os);
		}
		return null;
	}

	/**
	 * 過去アンケートリンク
	 *
	 * @return
	 * @throws ParseException
	 */
	@Execute(validator = false)
	public String enqLastLink() throws ParseException,IOException {

		String enqLastYear = enqResultListForm.enqLastYear;
		enqResultListForm.disFlg = false;

		enqResultListForm.enqYear = enqLastYear;
		List<Enquete> enqueteInfo = enqueteService.findEnqueteSummaryByYmd(enqLastYear);
		List<EnqResultDto> enqlist = new ArrayList<EnqResultDto>();
		for (int i = 0; i < enqueteInfo.size(); i++) {
			Enquete list = enqueteInfo.get(i);
			EnqResultDto dto = new EnqResultDto();
			dto.enqTitle = list.enqueteName;
			dto.enqId = String.valueOf(list.enqueteId);
			try {
				dto.enqId = URLEncoder.encode(keyHelper.encryptEnqueteId(list.enqueteId), "utf-8");
				dto.startDay = SinaburoUtil.convertStringDateToFormat(list.startDate, "/");
				dto.endDay = SinaburoUtil.convertStringDateToFormat(list.endDate, "/");
				enqlist.add(dto);
			} catch (Exception e) {
				logger.error("Encrypt failed. enqueteId：" + list.enqueteId + e.toString());
			}
		}
		session.setAttribute("enqRsultList", enqlist);

		return "/user/enqResultList/enqResultList.html";
	}
}
