package jp.agentec.sinaburocast.action.user;

import java.io.IOException;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SessionBindingListener;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.SinaburoConstant.UserType;
import jp.agentec.sinaburocast.common.annotation.NoLogin;
import jp.agentec.sinaburocast.common.exception.ExpectedException;
import jp.agentec.sinaburocast.common.exception.InvalidUrlException;
import jp.agentec.sinaburocast.common.exception.SystemException;
import jp.agentec.sinaburocast.common.util.AgtVelocityViewServlet;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.entity.PasswordMailSend;
import jp.agentec.sinaburocast.form.user.LoginForm;
import jp.agentec.sinaburocast.service.MemberService;
import jp.agentec.sinaburocast.service.PasswordMailSendService;

import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.h2.util.StringUtils;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;

/**
 * ログイン処理
 *
 * @author choi-c
 *
 */
@NoLogin
public class LoginAction extends AbstractAction {
	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	public LoginForm loginForm;

	public  MemberService memberService;

	public static final int sslMode = PropertyUtil.getInt("sslMode");

	public PasswordMailSendService passwordMailSendService;
	/**
	 * 初期表示
	 * @throws ExpectedException 
	 * @throws SystemException 
	 * @throws IOException 
	 */
	@Execute(validator = false)
	public String index() throws IOException, SystemException, ExpectedException {
		if(StringUtil.isNotBlank(loginForm.onetimeTicket)){
			PasswordMailSend passwordMailSend = passwordMailSendService.findByTicketAndMailType(loginForm.onetimeTicket,SinaburoConstant.MailType.MEMBER_REGIST);
			if(passwordMailSend != null){
				Member member = memberService.findById(passwordMailSend.memberId);
				loginForm.loginId=member.loginId;
				loginForm.password=member.password;
				auth();
				return "/user/userEdit/"; 
			}
		}
		
		AgtVelocityViewServlet avvs = new AgtVelocityViewServlet();
		if(avvs.getDevice().equals(SinaburoConstant.AccessType.PC)){
			// PCはトップページを使用する
			redirect("/");
			return null;
		} else {
			return "/user/login/login.html";
		}
	}

	/**
	 * ログイン押下
	 *
	 * @return
	 * @throws IOException
	 * @throws SystemException
	 * @throws ExpectedException
	 */
	@Execute(validator = false)
	public String auth() throws IOException, SystemException, ExpectedException {
		ActionMessages errors = loginValidate();

		if(!errors.isEmpty()){
			request.setAttribute(Globals.ERROR_KEY, errors);
			
			AgtVelocityViewServlet avvs = new AgtVelocityViewServlet();
			if(avvs.getDevice().equals(SinaburoConstant.AccessType.PC)){
				// PCはトップページを使用する
				return "/";
			} else {
				return "/user/login/login.html";
			}
		}

		renewSession();

		Member memberInfo = memberService.findByUserId(loginForm.loginId);

		setAuthenticatedToken(memberInfo,UserType.USER);

		if (sslMode == 1) {
			return "http://"  + request.getServerName() + request.getContextPath() + "/user/myPage/?redirect=true";
		}

//		setCookiePath("/em/user/");

		return "/user/myPage/";
	}

	/**
	 * ログアウト
	 *
	 * @return　ログイン画面
	 * @throws IOException
	 * @throws InvalidUrlException
	 */
	@Execute(validator = false)
	public String logout() throws IOException, InvalidUrlException {
		SessionBindingListener listener = (SessionBindingListener) session.getAttribute(SinaburoConstant.AttrKey.BINDING_LISTENER);
		if (listener != null) { // ログアウト押す前にタイムアウトしていた場合はnullになっている
			listener.setCauseAsLogout();
			session.setAttribute(SinaburoConstant.AttrKey.BINDING_LISTENER, listener);
		}

		redirect("/user/login/");
		session.invalidate(); // リダイレクト時にはまだセッションが有効である必要がある

		return null;
	}
	
	public ActionMessages loginValidate(){
		ActionMessages errors = new ActionMessages();
		
		//必須チェック
		if(StringUtil.isBlank(loginForm.loginId)){
			errors.add("loginId", new ActionMessage(SinaburoViewUtil.getMessage("errors.required.login",SinaburoViewUtil.getMessage("labels.loginId"))));
		}
		
		if(StringUtil.isBlank(loginForm.password)){
			errors.add("password", new ActionMessage(SinaburoViewUtil.getMessage("errors.required.login",SinaburoViewUtil.getMessage("labels.password"))));
		}
		
		if(!errors.isEmpty()){
			return errors;
		}
		
		Member memberInfo = memberService.findByUserId(loginForm.loginId);
		// ID、パスワードチェック
		if (memberInfo == null || !StringUtils.equals(memberInfo.password, loginForm.password)) {
			errors.add("password",new ActionMessage(SinaburoViewUtil.getMessage("errors.E002")));
		}

		return errors;
	}

}