package jp.agentec.sinaburocast.service;

import java.io.UnsupportedEncodingException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;
import org.seasar.extension.jdbc.AutoSelect;
import org.seasar.extension.jdbc.where.SimpleWhere;
import org.seasar.framework.container.annotation.tiger.Component;
import org.seasar.framework.container.annotation.tiger.InstanceType;
import org.seasar.framework.util.StringUtil;

import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.entity.Member;
import jp.agentec.sinaburocast.entity.MemberCardReceipt;
import jp.agentec.sinaburocast.form.admin.enquete.MemberCardReceiptSearchForm;

@Component(instance=InstanceType.SINGLETON)
public class MemberCardReceiptService extends AbstractService<MemberCardReceipt> {

	private final Logger logger = Logger.getLogger(getClass());
	
	private static final String ID_SEQ_NAME = "member_card_receipt_id_seq";
	/**
	 * IDを発行して、登録する。
	 *
	 */
	public int insertMemberCardReceipt(MemberCardReceipt memberCardReceipt, String insId) {
		memberCardReceipt.memberCardReceiptId = getSeqNextVal(Integer.class, ID_SEQ_NAME);
		return super.insert(memberCardReceipt, insId);
	}

	public MemberCardReceipt findById(Integer memberCardReceiptId) {
		return select().id(memberCardReceiptId).getSingleResult();
	}


	public List<MemberCardReceipt> findAllOrderById() {
		return select().orderBy("memberCardReceiptId asc").getResultList();
	}

	public List<MemberCardReceipt> findBymemberIdList(Integer memberId) {
		return select().where(new SimpleWhere().eq("memberId", memberId)).getResultList();
	}

	/**
	 * カード郵送申請情報を登録する
	 * @param member
	 */
	public void insertMemberCardReceipt(Member member) {
		MemberCardReceipt memberCardReceipt = new MemberCardReceipt();
		memberCardReceipt.memberId = member.memberId;
		memberCardReceipt.applyDate = SinaburoUtil.getTimestamp();
		insertMemberCardReceipt(memberCardReceipt, member.loginId);
	}

	/**
	 * カード郵送申請情報取得（画面用）
	 * @param memberCardReceiptSearchForm
	 * @return ArrayList<MemberCardReceipt>
	 */
	public ArrayList<MemberCardReceipt> findAllByPaging(MemberCardReceiptSearchForm memberCardReceiptSearchForm){

		//件数取得
		AutoSelect<MemberCardReceipt> counter = this.getAutoSelectForFindAllByPaging(memberCardReceiptSearchForm);
		memberCardReceiptSearchForm.setPaging(null, memberCardReceiptSearchForm.pageNo, counter.getCount());

		//結果取得
		return (ArrayList<MemberCardReceipt>)this.getAutoSelectForFindAllByPaging(memberCardReceiptSearchForm).orderBy("applyDate,memberId")
				.offset(memberCardReceiptSearchForm.getOffSet(""))
				.limit(memberCardReceiptSearchForm.getLimit(""))
				.getResultList();
	}
	
	/**
	 * 会員IDと月で検索し、データの有無を判定する
	 * @return
	 */
	public boolean findByMemberIdAndMonth(Integer memberId, String year, String month) {
		List<MemberCardReceipt> resultList = findBymemberIdList(memberId);
		
		if(resultList.size() == 0) {
			return false;
		}
		
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM");
		for(MemberCardReceipt memberCardReceipt : resultList) {
			String [] applyYearMonth = sdf.format(memberCardReceipt.applyDate).split("/");
			if(year.equals(applyYearMonth[0]) && month.equals(applyYearMonth[1])) {
				return true;
			}
		}
		return false;
	}

	/**
	 * カード郵送申請情報を検索するための、AutoSelectを返す。
	 * @param memberCardReceiptSearchForm
	 * @return AutoSelect<MemberCardReceipt>
	 */
	private AutoSelect<MemberCardReceipt> getAutoSelectForFindAllByPaging(MemberCardReceiptSearchForm memberCardReceiptSearchForm){

		Date fromDay = null;
		Date toDay = null;

		if (!StringUtil.isBlank(memberCardReceiptSearchForm.fromDay)){
			fromDay=SinaburoUtil.convertStringToTimestamp(memberCardReceiptSearchForm.fromDay,"yyyy-MM-dd");
		}
		if (!StringUtil.isBlank(memberCardReceiptSearchForm.toDay)){
			toDay=SinaburoUtil.convertStringToTimestamp(SinaburoUtil.getNextDay(memberCardReceiptSearchForm.toDay),"yyyy-MM-dd");
		}

		return select().innerJoin("member").innerJoin("member.prefecture").where(new SimpleWhere()
		.ge("applyDate", fromDay)
		.lt("applyDate", toDay)
		.eq("member.validFlg", SinaburoConstant.MemberValidFlg.VALID)
		.eq("member.delFlg", SinaburoConstant.MemberDelFlg.NOT_DEL));
	}

	/**
	 * CSVファイル出力カード郵送申請情報取得
	 * @param memberCardReceiptSearchForm
	 * @return String カード郵送申請情報文字列
	 */
	public String getMemberCardReceiptForCsv(MemberCardReceiptSearchForm memberCardReceiptSearchForm) throws UnsupportedEncodingException, Exception{

		StringBuffer str = new StringBuffer();
		str.append("\"申請日\",\"会員ID\",\"氏名\",\"郵便番号\",\"都道府県\",\"市区町村\",\"町名・番地\",\"建物名\",\"電話番号\""+"\r\n");

		List<MemberCardReceipt> memberCardReceiptList = getMemberCardReceiptByFromDayAndToDay(memberCardReceiptSearchForm);

    	for (MemberCardReceipt memberCardReceipt : memberCardReceiptList) {

    		//申請日
    		str.append("\"" + SinaburoUtil.dateToString2(memberCardReceipt.applyDate) + "\",");

    		//会員ID
    		str.append("\"" + memberCardReceipt.memberId +"\",");

    		//氏名
    		str.append("\"" + memberCardReceipt.member.firstName + " " + memberCardReceipt.member.lastName + "\",");

    		//郵便番号
    		str.append("\"" + memberCardReceipt.member.zipCode + "\",");

    		//都道府県
    		str.append("\"" + memberCardReceipt.member.prefecture.prefecture + "\",");

    		//市区町村
    		str.append("\"" + memberCardReceipt.member.cityName + "\",");

    		//町名・番地
    		str.append("\"" + memberCardReceipt.member.areaName + "\",");

    		//建物名
    		str.append("\"" + replaceNullWithBlank(memberCardReceipt.member.buildingName) + "\",");
    		
    		//電話番号
    		str.append("\"" + memberCardReceipt.member.telno + "\"");

    		str.append("\r\n");

    	}
    	return str.toString();

	}

	/**
	 * null値を空白に置き換え
	 */
	private String replaceNullWithBlank(String str) {
		if(str != null) {
			return str;
		}
		return "";
	}
	
	/**
	 * カード郵送申請情報取得（CSV出力用）
	 * @param memberCardReceiptSearchForm
	 * @return List<MemberCardReceipt>
	 */
	private List<MemberCardReceipt> getMemberCardReceiptByFromDayAndToDay(MemberCardReceiptSearchForm memberCardReceiptSearchForm){

		Date fromDay = null;
		Date toDay = null;

		if (!StringUtil.isBlank(memberCardReceiptSearchForm.fromDay)){
			logger.info("memberCardReceiptSearchForm.fromDay:" + memberCardReceiptSearchForm.fromDay);
			fromDay = SinaburoUtil.convertStringToTimestamp(memberCardReceiptSearchForm.fromDay,"yyyy-MM-dd");
		}
		if (!StringUtil.isBlank(memberCardReceiptSearchForm.toDay)){
			logger.info("memberCardReceiptSearchForm.toDay:" + memberCardReceiptSearchForm.toDay);
			toDay = SinaburoUtil.convertStringToTimestamp(SinaburoUtil.getNextDay(memberCardReceiptSearchForm.toDay),"yyyy-MM-dd");
		}

		return select().innerJoin("member").innerJoin("member.prefecture").where(new SimpleWhere()
		.ge("applyDate", fromDay)
		.lt("applyDate", toDay)
		.eq("member.validFlg", SinaburoConstant.MemberValidFlg.VALID)
		.eq("member.delFlg", SinaburoConstant.MemberDelFlg.NOT_DEL))
		.orderBy("applyDate,memberId")
		.getResultList();
	}


}